@@ -1,5 +1,5 @@
 #!/bin/bash
-# lesspipe.sh, a preprocessor for less (version 2.02)
+# lesspipe.sh, a preprocessor for less (version 2.05)
 # Author: Wolfgang Friebel (wp.friebel AT gmail.com)
 #( [[ -n 1 && -n 2 ]] ) > /dev/null 2>&1 || exec zsh -y --ksh-arrays -- "$0" ${1+"$@"}
 
@@ -40,7 +40,7 @@ filetype () {
 			ftype=pptx ;;
 		openxmlformats-officedocument.spreadsheetml.sheet)
 			ftype=xlsx ;;
-		oasis.opendocument.text)
+		oasis.opendocument.text*)
 			ftype=odt ;;
 		oasis.opendocument.spreadsheet)
 			ftype=ods ;;
@@ -85,6 +85,8 @@ filetype () {
 				ftype=csr ;;
 			*PEM\ certificate)
 				ftype=csr ;;
+			*Microsoft\ OOXML)
+				ftype=docx ;;
 			Apple\ binary\ property\ list)
 				ftype=plist ;;
 			PGP\ *ncrypted*|GPG\ encrypted*)
@@ -289,6 +291,12 @@ get_unpack_cmd () {
 			has_cmd zstd && cmd=(zstd -cdqM1073741824 "$2") && return ;;
 		lz4)
 			has_cmd lz4 && cmd=(lz4 -cdq "$2") && return ;;
+		xlsx)
+			has_cmd in2csv && cmd=(in2csv -f xlsx "$2") && return
+			has_cmd excel2csv && cmd=(istemp excel2csv "$2") && return ;;
+		ms-excel)
+			has_cmd in2csv && cmd=(in2csv -f xls "$2") && return
+			has_cmd xls2csv && cmd=(istemp xls2csv "$2") && return ;;
 	esac
 	# convert into utf8
 
@@ -367,10 +375,10 @@ analyze_args () {
 	# return if we want to watch growing files
 	[[ $lessarg == *less\ *\ +F\ * || $lessarg == *less\ *\ : ]] && exit 0
 	# color is set when calling less with -r or -R or LESS contains that option
-	lessarg="$LESS $lessarg"
-	lessarg=`echo $lessarg|sed 's/-[a-zA-Z]*[rR]/-r/'`
+	lessarg="l $LESS $lessarg"
+	lessarg=`echo $lessarg|sed 's/ -[a-zA-Z]*[rR]/ -r/'`
 	has_cmd tput && colors=$(tput colors) || colors=0
-	if [[ $colors -ge 8 && $lessarg == *-[rR]* ]]; then
+	if [[ $colors -ge 8 && $lessarg == *\ -[rR]* ]]; then
 		COLOR="--color=always"
 	else
 		COLOR="--color=auto"
@@ -390,11 +398,12 @@ has_colorizer () {
 		[[ -z $prog || $prog == $i ]] && has_cmd $i && prog=$i
 	done
 	[[ "$2" =~ ^[0-9]*$ ]] && opt= || opt=" -l $2"
+	ext=${fileext##*.}
 	case $prog in
 		bat|batcat)
 			# only allow an explicitly requested language
-			opt=" -l $fileext"
-			{ [[ -n $fileext ]] && $prog $opt /dev/null; } || opt=
+			opt=" -l $ext"
+			{ [[ -n $ext ]] && $prog $opt /dev/null; } || opt=
 			opt="$opt $COLOR" ;;
 		pygmentize)
 				[[ -n $LESSCOLORIZER && $LESSCOLORIZER =~ pygmentize\ \ *-O\ *style=[a-z]* ]] && prog=$LESSCOLORIZER
@@ -403,15 +412,14 @@ has_colorizer () {
 		source-highlight)
 			prog="source-highlight --failsafe -f esc"
 			[[ -z $arg ]] && arg=/dev/stdin
-			[[ -n $opt ]] && opt=" -s $2 -i" || opt=" -i" ;;
-		code2color)
-			[[ -n $opt ]] && opt=" -l $2" ;;
-		vimcolor)
-			;;
+			[[ -n "$opt" && -n "$2" ]] && opt=" -s $2 -i" || opt=" -i" ;;
+		code2color|vimcolor)
+			opt=
+			[[ -n "$fileext" ]] && opt=" -l $fileext" ;;
 		*)
 			return ;;
 	esac
-	echo "$prog$opt $arg"
+	echo "$prog$opt"
 }
 
 isfinal () {
@@ -444,7 +452,8 @@ isfinal () {
 			# filename needs to end in .class
 			has_cmd procyon && t=$t.class && cat "$1" > $t && cmd=(procyon "$t") ;;
 		markdown)
-			{ has_cmd mdcat && cmd=(mdcat "$1"); } ||
+			[[ $COLOR = *always ]] && mdopt= || mdopt=-c
+			{ has_cmd mdcat && cmd=(mdcat $mdopt "$1"); } ||
 			{ has_cmd pandoc && cmd=(pandoc -t plain "$1"); } ;;
 		docx)
 			{ has_cmd pandoc && cmd=(pandoc -f docx -t plain "$1"); } ||
@@ -458,13 +467,11 @@ isfinal () {
 					-o $t2" "$1" && cmd=(pandoc -f markdown -t plain "$t2"); }; } ||
 			{ has_cmd libreoffice && has_htmlprog && cmd=(isoffice "$1" ppt); } ;;
 		xlsx)
-			{ has_cmd in2csv && cmd=(in2csv -f xlsx "$1"); } ||
 			{ has_cmd xlscat && cmd=(istemp xlscat "$1"); } ||
-			{ has_cmd excel2csv && cmd=(istemp excel2csv "$1"); } ||
 			{ has_cmd libreoffice && has_htmlprog && cmd=(isoffice "$1" xlsx); } ;;
 		odt)
-			{ has_cmd pandoc && cmd=(pandoc -f odt -t plain "$1"); } ||
 			{ has_cmd odt2txt && cmd=(istemp odt2txt "$1"); } ||
+			{ has_cmd pandoc && cmd=(pandoc -f odt -t plain "$1"); } ||
 			{ has_cmd libreoffice && cmd=(isoffice2 "$1"); } ;;
 		odp)
 			{ has_cmd libreoffice && has_htmlprog && cmd=(isoffice "$1" odp); } ;;
@@ -481,8 +488,6 @@ isfinal () {
 			{ has_cmd broken_catppt && cmd=(istemp catppt "$1"); } ||
 			{ has_cmd libreoffice && has_htmlprog && cmd=(isoffice "$1" ppt); } ;;
 		ms-excel)
-			{ has_cmd in2csv && cmd=(in2csv -f xls "$1"); } ||
-			{ has_cmd xls2csv && cmd=(istemp xls2csv "$1"); } ||
 			{ has_cmd libreoffice && has_htmlprog && cmd=(isoffice "$1" xls); } ;;
 		ooffice1)
 			{ has_cmd sxw2txt && cmd=(istemp sxw2txt "$1"); } ||
@@ -509,7 +514,7 @@ isfinal () {
 			{ { has_cmd pod2text && cmd=(pod2text "$1"); } ||
 			{ has_cmd perldoc && cmd=(istemp perldoc "$1"); }; } ;;
 		pst)
-			has_cmd perl && perl -MStorable=retrieve -MData::Dumper -e '$Data::Dumper::Indent=1;print Dumper retrieve shift' "$1" ;;
+			has_cmd perl && cmd=(perl -MStorable=retrieve -MData::Dumper -e '$Data::Dumper::Indent=1;print Dumper retrieve shift' "$1") ;;
 		hdf)
 			{ has_cmd h5dump && cmd=(istemp h5dump "$1"); } ||
 			{ has_cmd ncdump && cmd=(istemp ncdump "$1"); } ;;
@@ -528,8 +533,11 @@ isfinal () {
 		mp3)
 			has_cmd id3v2 && cmd=(istemp "id3v2 --list" "$1") ;;
 		log)
-			has_cmd ccze && cat "$1" | ccze -A
+			has_cmd ccze && [[ $COLOR = *always ]] && cat "$1" | ccze -A
 			return ;;
+		csv)
+			{ has_cmd csvlook && cmd=(csvlook "$1"); } ||
+			{ has_cmd pandoc && cmd=(pandoc -f csv -t plain "$1"); } ;;
 	esac
 	fi
 	# not a specific file format
@@ -559,7 +567,7 @@ isfinal () {
 		[[ -z "$fext" ]] && fext=$(fileext "$fileext")
 		fext=${fext##*/}
 		[[ -z $colorizer ]] && colorizer=$(has_colorizer "$1" "$fext")
-		[[ -n $colorizer && $fcat != binary ]] && $colorizer && return
+		[[ -n $colorizer && $fcat != binary ]] && $colorizer "$1" && return
 		# if fileext set, we need to filter to get rid of .fileext
 		[[ -n $fileext || "$1" == - || "$1" == $t ]] && cat "$1"
 	fi
@@ -715,7 +723,7 @@ setopt sh_word_split 2>/dev/null
 PATH=$PATH:${0%%/lesspipe.sh}
 # the current locale in lowercase (or generic utf-8)
 locale=$(locale|grep LC_CTYPE|sed 's/.*"\(.*\)"/\1/') || locale=en_US.UTF-8
-lclocale=$(echo ${locale##*.}|sed 's/\(.*\)/\L\1/')
+lclocale=$(echo ${locale##*.}|tr '[A-Z]' '[a-z]')
 
 sep=:					# file name separator
 altsep==				# alternate separator character
@@ -751,11 +759,10 @@ if [[ -z "$1" ]]; then
 		echo "export LESSOPEN"
 	fi
 else
-	if [ -x ~/.lessfilter ]; then
-		~/.lessfilter "$1"
-		if [ $? -eq 0 ]; then
-			exit 0
-		fi
+	if [ -x "${HOME}/.lessfilter" ]; then
+		"${HOME}/.lessfilter" "$1" && exit 0
+	elif has_cmd lessfilter; then
+		lessfilter "$1" && exit 0
 	fi
 	show "$@"
 fi
